/******************************************************************************
 *
 *	Papillon
 *	color edition
 *	ver 1.1.0
 *	2008.04
 *	ChoiKyun
 *
 *****************************************************************************/
#include <PalmOS.h>
#include <PalmOSGlue.h>
#include <SonyCLIE.h>
#include "defrsc.h"

#define GLOBAL_VALUE_DEFINE
#include "game.h"



/******************************************************************************
 *	vg^Cv
 *****************************************************************************/
Boolean check_hard_key( EventPtr );
void get_pen_input( void );
void check_enter( EventPtr );
Int32 set_event_interval( void );
void init_main_form( FormPtr );
void make_bitmap_offscreen( void );
Err rom_check( UInt32, UInt16 );
void *get_object_ptr( UInt16 );
Boolean main_form_command( UInt16 );
Boolean main_form_handler( EventPtr );
Boolean app_handle_event( EventPtr );
void event_loop( void );
Err app_start( void );
void app_stop( void );
UInt32 my_app_main( UInt16, MemPtr, UInt16 );



/************************************************
 *	n[hL[
 ***********************************************/
void
get_hard_key( void )
{
  Key.keep = Key.single;
  Key.keep ^= 0xffffffff;

  Key.single = KeyCurrentState();
  Key.keep &= Key.single;
}



/********************************************************************
 *	X^[eBbNȊ֐Q
 *******************************************************************/

/************************************************
 *	\[Xrbg}bvItXN[쐬
 ***********************************************/
static void
make_bitmap_offscreen( void )
{
  int i;
  WinHandle save;
  UInt16 error;
  MemHandle h;
  BitmapPtr p;


  save = WinGetDrawWindow();

  // LN^@ItXN[쐬
  for( i = 0; i <= MAX_BMP; i++ )
    {
      h = DmGetResource( bitmapRsc, BMP_BASE_ID + i );
      p = MemHandleLock( h );

      Bmp.handle[i] = WinCreateOffscreenWindow( p ->width, p ->height,
                                                screenFormat, &error );
      ErrFatalDisplayIf( error, "Error loading images" );

      WinSetDrawWindow( Bmp.handle[i] );
      WinDrawBitmap( p, 0, 0 );

      MemPtrUnlock( p );
      DmReleaseResource( h );
    }

  WinSetDrawWindow( save );
}



/************************************************
 *	n[hL[̃`FbN
 ***********************************************/
static Boolean
check_hard_key( EventPtr event_p )
{
  Boolean handled = false;


  switch( event_p ->eType )
    {
    case keyDownEvent:
      if( TxtGlueCharIsVirtual( event_p ->data.keyDown.modifiers,
                                event_p ->data.keyDown.chr) )
        {
          switch( event_p ->data.keyDown.chr )
            {
            case vchrHard1:
            case vchrHard2:
            case vchrHard3:
            case vchrHard4:
              Game.pause = false;	// |[Y
              handled = true;
              break;

            case vchrPageDown:
            case vchrPageUp:
              Game.pause = true;	// |[Y
              handled = true;
              break;
            }
        }
    }

  return handled;
}



/************************************************
 *	ANeBu`FbN
 ***********************************************/
static void
check_enter( EventPtr event_p )
{
  switch( event_p ->eType )
    {
    case winEnterEvent:
      if( event_p ->data.winEnter.enterWindow == (WinHandle) FrmGetFormPtr( MainForm )
          && event_p ->data.winEnter.enterWindow == (WinHandle) FrmGetFirstForm() )
        Game.active = true;
      break;

    case winExitEvent:
      if( event_p ->data.winExit.exitWindow == (WinHandle) FrmGetFormPtr( MainForm ) )
        Game.active = false;
      break;
    }
}



/************************************************
 *	ROM̃o[W`FbN
 *	PalmOS 3.00ȍ~H
 ***********************************************/
static Err
rom_check(UInt32 requiredVersion, UInt16 launchFlags)
{
  UInt32 rom_ver;

  // See if we're on in minimum required version of the ROM or later.
  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &rom_ver);

  if( rom_ver < requiredVersion )
    {
      if( ( launchFlags & ( sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp ) )
          == ( sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp ) )
        {
          FrmAlert( RomIncompatibleAlert );

          // Palm OS 1.0 will continuously relaunch this app unless we switch to 
          // another safe one.
          if( rom_ver < ourMinVersion )
            {
              AppLaunchWithCommand( sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL );
            }
        }

      return sysErrRomIncompatible;
    }

  return errNone;
}



/************************************************
 *	Cxg[ṽEFCgݒ
 ***********************************************/
static Int32
set_event_interval( void )
{
  Int32 wait;


  wait = Timer.interval + DEF_EVENT_TICK - TimGetTicks();
  if( wait < 0 ) wait = 0;

  return wait;
}



#if 0
/************************************************
 *	tH[̃IuWFNg̃|C^擾
 ***********************************************/
static void *
get_object_ptr( UInt16 object_ID )
{
  FormPtr frm_p;


  frm_p = FrmGetActiveForm();
  return FrmGetObjectPtr( frm_p, FrmGetObjectIndex( frm_p, object_ID ) );
}
#endif



/************************************************
 *	CtH[j[R}h
 ***********************************************/
static Boolean
main_form_command( UInt16 command )
{
  Boolean handled = false;
  FormPtr frm_p;


  switch( command )
    {
    case MainOptionsNewGame:
      MenuEraseStatus( 0 );

      // j[Q[
      clear_screen();
      init_demo();
      Timer.interval = TimGetTicks();

      handled = true;
      break;

    case MainOptionsScore:
      // XRALO
      show_ranking();

      Timer.interval = TimGetTicks();
      handled = true;
      break;

    case MainOptionsAboutPapillon:
      // AoEgʂ̕\
      MenuEraseStatus( 0 );
      frm_p = FrmInitForm( AboutForm );
      FrmDoDialog( frm_p );
      FrmDeleteForm( frm_p );

      Timer.interval = TimGetTicks();
      handled = true;
      break;

    case MainOptionsHelp:
      // wv
      FrmHelp( key_helpString );
      Timer.interval = TimGetTicks();
      handled = true;
      break;
    }

  return handled;
}



/************************************************
 *	CtH[Cxgnh
 ***********************************************/
static Boolean
main_form_handler( EventPtr event_p )
{
  Boolean handled = false;
  FormPtr frm_p;


  switch( event_p ->eType )
    {
    case nilEvent:
      handled = true;
      break;

    case menuEvent:
      return main_form_command( event_p ->data.menu.itemID );

    case frmOpenEvent:
      frm_p = FrmGetActiveForm();
      FrmDrawForm( frm_p );
      handled = true;
      break;

    case penUpEvent:
      Pen.down = false;
      Pen.move = false;
      Pencil.move = false;

      handled = true;
      break;

    case penMoveEvent:
      Pen.move = true;
      Pen.x = event_p -> screenX + BACK_W_BLANK;
      Pen.y = event_p -> screenY + BACK_H_BLANK;

      handled = true;
      break;

    case penDownEvent:
      Pen.down = true;
      Pen.x = event_p -> screenX + BACK_W_BLANK;
      Pen.y = event_p -> screenY + BACK_H_BLANK;

      if( (UInt16) Pen.x - (Pencil.x-4) < PENCIL_WIDTH+8
          && (UInt16) Pen.y - (Pencil.y-4) < PENCIL_HEIGHT+8 )
        {
          Pencil.move = true;
          Pencil.dx = Pen.x - Pencil.x;
          Pencil.dy = Pen.y - Pencil.y;
//         set_se( down );
        }

      if( Pen.y > BACK_H_BLANK )
        Game.pause = false;		// |[Y
      handled = true;
      break;
    }

  return handled;
}



/************************************************
 *	tH[̃[h
 ***********************************************/
static Boolean
app_handle_event( EventPtr event_p )
{
  UInt16 form_id;
  FormPtr frm_p;


  if( event_p ->eType == frmLoadEvent )
    {
      form_id = event_p ->data.frmLoad.formID;
      frm_p = FrmInitForm( form_id );
      FrmSetActiveForm( frm_p );

      switch( form_id )
        {
        case MainForm:
          FrmSetEventHandler( frm_p, main_form_handler );
          break;
        }

      return true;
    }

  return false;
}



/************************************************
 *	Cxg[v
 ***********************************************/
static void
event_loop( void )
{
  UInt16 error;
  EventType event;


  do
    {
      // Cxg擾
      EvtGetEvent( &event, set_event_interval() );

      // ANeBu
      check_enter( &event );
      // Q[̃C
      play_game();

      // n[hL[ŃXLbv
      if( !check_hard_key( &event ) )
        if( SysHandleEvent( &event ) ) continue;

      if( !MenuHandleEvent( 0, &event, &error ) )
        if( !app_handle_event( &event ) )
          FrmDispatchEvent( &event );
    }
  while( event.eType != appStopEvent );
}



/************************************************
 *	AvJn
 ***********************************************/
static Err
app_start( void )
{
  Int16 ver;
  UInt16 size;
  GamePrefType pref;


  if( init_screen() )
    {
      FrmAlert( NonSupportedScreenAlert );
      return -1;
    }

  make_bitmap_offscreen();

  SysRandom ( TimGetTicks() );
  AutoOff.seconds = SysSetAutoOffTime( AUTO_OFF_TIME );

  Sound.on = true;
  Sound.volume = PrefGetPreference( prefGameSoundVolume );

  // Xg[Nobt@
  Stroke.buf_p = (PointType *) MemPtrNew( sizeof(PointType) * STROKE_MAX );
  ErrFatalDisplayIf( !Stroke.buf_p, "Error Allocating Buffer" );
  // Gobt@
  Tefu.buf_p = (EnemyType *) MemPtrNew( sizeof(EnemyType) * ENEMY_MAX );
  ErrFatalDisplayIf( !Tefu.buf_p, "Error Allocating Buffer" );

  // nCXRA
  init_hiscore();

  // vt@X擾
  size = sizeof( GamePrefType );
  ver = PrefGetAppPreferences( appFileCreator, appPrefID, &pref, &size, true );
  if( ver == noPreferenceFound
      || ver != appPrefVersionNum
      || size != sizeof( GamePrefType) )
    {
      // 
      init_hiscore();
    }
  else
    {
      // vt@X̃Rs[
      MemMove( &Rank, &pref, sizeof( RankingType ) );
      Sound.on = pref.sound;
    }

  init_demo();
  Timer.interval = TimGetTicks();

  return errNone;
}



/************************************************
 *	AvI
 ***********************************************/
static void
app_stop( void )
{
  int i;
  UInt16 hr_ref_num;
  GamePrefType pref;

  // vt@X̃Rs[
  MemMove( &pref, &Rank, sizeof( RankingType ) );
  pref.sound = Sound.on;
  PrefSetAppPreferences( appFileCreator, appPrefID, appPrefVersionNum,
                         &pref, sizeof( GamePrefType ), true );

  for( i = 0; i <= MAX_BMP; i++ )
    WinDeleteWindow( Bmp.handle[i], false );
  // wi
//  WinDeleteWindow( Bg.handle, false );

  // obt@
  MemPtrFree( Stroke.buf_p );
  MemPtrFree( Tefu.buf_p );

  // obNXN[
  WinDeleteWindow( Screen.back_h, false );

  // ʂ߂
  if( ! SysLibFind( sonySysLibNameHR, &hr_ref_num ) )
    {
      HROpen( hr_ref_num );
      HRWinScreenMode( hr_ref_num,
                       winScreenModeSet,
                       &Screen.save.width,
                       &Screen.save.height,
                       &Screen.save.depth,
                       NULL );
      HRClose( hr_ref_num );
    }
  else
    {
      WinScreenMode( winScreenModeSet,
                     &Screen.save.width,
                     &Screen.save.height,
                     &Screen.save.depth,
                     NULL );
    }

  SysSetAutoOffTime( AutoOff.seconds );
  FrmCloseAllForms();
}



/************************************************
 *	C
 ***********************************************/
static UInt32
my_app_main( UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags )
{
  Err error;

  // ROM`FbN OS3.5ȍ~
  error = rom_check( PalmRomVer350, launchFlags );
  if( error ) return error;

  switch( cmd )
    {
    case sysAppLaunchCmdNormalLaunch:
      error = app_start();
      if( error ) return error;

      FrmGotoForm( MainForm );
      event_loop();
      app_stop();
      break;

    default:
      break;
    }

  return errNone;
}



/************************************************
 *	Gg[
 ***********************************************/
UInt32
PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags )
{
  return my_app_main( cmd, cmdPBP, launchFlags );
}



